import ago
from datetime import datetime
import logging
import memcache
import requests
import simplejson
import urlparse

from Config import Config


class Untappd(object):
    API_URL = "https://api.untappd.com/v4"

    @classmethod
    def api_request(cls, endpoint, params=None):
        if params is None:
            params = dict()

        params["client_id"] = Config.get("untappd_api_id")
        params["client_secret"] = Config.get("untappd_api_secret")

        url = urlparse.urljoin(cls.API_URL, endpoint)
        req = requests.get(url, params=params)
        req.raise_for_status()
        return req.json()


class Beer(object):
    memcache = memcache.Client(["127.0.0.1:11211"])

    def to_dict(self):
        return {
            "beer_id": self.beer_id,
            "beer_name": self.beer_name,
            "beer_style": self.beer_style,
            "beer_label": self.beer_label,
            "description": self.description,
            "abv": self.abv,
            "brewery_name": self.brewery_name,
            "brewery_loc": self.brewery_loc,
            "brewery_label": self.brewery_label,
            }

    def to_json(self):
        return simplejson.dumps(self.to_dict())

    @classmethod
    def new_from_api_response(cls, beer, brewery=None):
        if brewery is None and "brewery" in beer:
            brewery = beer["brewery"]

        obj = cls()
        obj.beer_id = beer["bid"]
        obj.beer_name = beer["beer_name"]
        obj.beer_style = beer["beer_style"]
        obj.beer_label = beer["beer_label"]
        obj.description = beer["beer_description"]
        obj.abv = beer["beer_abv"]
        obj.brewery_name = brewery["brewery_name"]
        obj.brewery_label = brewery["brewery_label"]
        obj.brewery_loc = "{}, {}, {}".format(
            brewery["location"]["brewery_city"],
            brewery["location"]["brewery_state"],
            brewery["country_name"],
            )

        return obj

    @classmethod
    def new_from_id(cls, beer_id):
        endpoint = "/v4/beer/info/{}".format(beer_id)

        data = cls.memcache.get(str(beer_id))
        if data is None:
            data = Untappd.api_request(endpoint)
            cls.memcache.set(str(beer_id), data, 60 * 60 * 24)

        beer = data["response"]["beer"]

        obj = cls.new_from_api_response(beer)
        return obj

    @classmethod
    def search(cls, search_string):
        endpoint = "/v4/search/beer"
        data = Untappd.api_request(endpoint, {"q": search_string})
        beers = []

        for item in data["response"]["beers"]["items"]:
            beers.append(cls.new_from_api_response(beer=item["beer"], brewery=item["brewery"]))

        return beers


class Checkin(object):
    latest_checkins = None

    @property
    def beer(self):
        if not hasattr(self, "_beer"):
            self._beer = Beer.new_from_id(self.beer_id)

        return self._beer

    @property
    def time_since(self):
        try:
            delta = datetime.utcnow() - datetime.strptime(self.created_at, "%a, %d %b %Y %H:%M:%S +0000")
            return ago.human(delta, precision=2)
        except AttributeError as e:
            return ""
        except:
            raise

    @classmethod
    def new_from_api_response(cls, response):
        obj = cls()

        obj.checkin_id = response["checkin_id"]
        obj.user_name = response["user"]["first_name"]
        obj.user_avatar = response["user"]["user_avatar"]
        obj.created_at = response["created_at"]
        obj.comment = response["checkin_comment"]
        obj.beer_id = response["beer"]["bid"]

        return obj

    @classmethod
    def get_latest(cls):
        endpoint = "/v4/venue/checkins/{}".format(Config.get("untappd_venue_id"))

        try:
            data = Untappd.api_request(endpoint)
        except Exception as e:
            logging.warning("Couldn't update checkin details: {}".format(e))
            return cls.latest_checkins

        checkins = []
        for item in data["response"]["checkins"]["items"]:
            checkins.append(cls.new_from_api_response(item))

        cls.latest_checkins = checkins
        return checkins



import logging
import requests
import urlparse

from kegmeter.common import Config

class DBClient(object):
    @classmethod
    def web_host(cls):
        return "{}:{}".format(Config.get("web_remote_host"), Config.get("web_remote_port"))

    @classmethod
    def get_taps(cls):
        url = urlparse.urljoin(cls.web_host(), "json")
        req = requests.get(url)
        return req.json()

    @classmethod
    def update_amount_poured(cls, tap_id, pulses):
        logging.debug("Sending update for tap {}, {} pulses".format(tap_id, pulses))

        url = urlparse.urljoin(cls.web_host(), "update")
        requests.post(url, params={
                "update_secret": Config.get("update_secret"),
                "tap_id": tap_id,
                "pulses": pulses,
                })

    @classmethod
    def update_temperature(cls, sensor_id, deg_c):
        logging.debug("Sending update for temperature sensor {}, {} degrees".format(sensor_id, deg_c))

        url = urlparse.urljoin(cls.web_host(), "update")
        requests.post(url, params={
                "update_secret": Config.get("update_secret"),
                "sensor_id": sensor_id,
                "deg_c": deg_c,
                })



import os
from setuptools import setup, find_packages

setup(
    name="kegmeter-common",
    version="0.1",
    author="OmniTI Computer Consulting, Inc.",
    author_email="hello@omniti.com",
    license="MIT",
    packages=["kegmeter.common"],
    package_dir={"kegmeter.common": "."},
    install_requires=[
        "ago >= 0.0.6",
        "oauth2client >= 1.3.1",
        "pysqlite >= 2.6.3",
        "python-memcached >= 1.53",
        "requests >= 1.2.3",
        "simplejson >= 3.6.5",
        "tornado >= 4.0.2",
        ],
    scripts=[
        "kegmeter_web.py",
        ],
    )



import os
import simplejson

class Config(object):
    settings = None
    base_dir = "/opt/kegmeter"
    config_file = os.path.join(base_dir, "etc", "settings.json")

    @classmethod
    def get(cls, item):
        cls.parse()
        if item in cls.settings:
            return cls.settings[item]

    @classmethod
    def parse(cls):
        if cls.settings is not None:
            return

        with open(cls.config_file, "r") as fh:
            contents = fh.read()
            cls.settings = simplejson.loads(contents)



import os
from setuptools import setup, find_packages

setup(
    name="kegmeter-common",
    description="Kegmeter libraries used by both the app and the webserver",
    url="https://github.com/Dennovin/kegmeter",
    version="0.21",
    author="OmniTI Computer Consulting, Inc.",
    author_email="hello@omniti.com",
    license="MIT",
    namespace_packages=["kegmeter"],
    packages=find_packages(),
    install_requires=[
        "ago >= 0.0.6",
        "oauth2client >= 1.3.1",
        "python-memcached >= 1.53",
        "requests >= 1.2.3",
        "simplejson >= 3.6.5",
        ],
    )
